package stream.dashboard;

import java.io.Serializable;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.Data;
import stream.ProcessContext;
import stream.dashboard.renderer.MessageRenderer;

public class Messages extends Widget {

	static Logger log = LoggerFactory.getLogger(Messages.class);

	/** The unique class ID */
	private static final long serialVersionUID = -6439498544151510135L;

	final MessageList listModel = new MessageList();
	final JList<Message> list = new JList<Message>(listModel);

	String key = null;
	int max = 10;

	public Messages() {
	}

	@Override
	public void init(ProcessContext ctx) throws Exception {
		super.init(ctx);
		setTitle("Messages");
		list.setCellRenderer(new MessageRenderer());
		JScrollPane scroll = new JScrollPane(list);
		scroll.setBorder(null);
		content.add(list);
	}

	/**
	 * @see stream.Processor#process(stream.Data)
	 */
	public Data process(Data input) {
		if (input == null || key == null)
			return input;

		Serializable value = input.get(key);
		log.info("{} = {}", key, value);
		if (value == null) {
			return input;
		}

		String msg = value.toString();
		log.info("Message is: {}", msg);
		listModel.addElement(new Message(msg));
		while (listModel.getSize() > max) {
			listModel.remove(0);
		}
		log.info("Model has {} messages", list.getModel().getSize());
		list.validate();
		return input;
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public int getMax() {
		return max;
	}

	public void setMax(int max) {
		this.max = max;
	}

	public static class MessageList extends DefaultListModel<Message> {
		private static final long serialVersionUID = -2221851153312421414L;
	}
}
