/**
 * 
 */
package stream.dashboard;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.general.PieDataset;

import stream.data.Statistics;

/**
 * @author chris
 * 
 */
public class PieDatasetAdapter implements PieDataset {

	final ArrayList<String> keys = new ArrayList<String>();
	final List<DatasetChangeListener> listener = new ArrayList<DatasetChangeListener>();
	final Statistics data = new Statistics();
	DatasetGroup group;

	ColorMapping colorMapping = new DefaultColorMapping();
	Map<String, Color> colors = new LinkedHashMap<String, Color>();

	/**
	 * @see org.jfree.data.KeyedValues#getKey(int)
	 */
	@SuppressWarnings("rawtypes")
	public Comparable getKey(int index) {
		return keys.get(index);
	}

	/**
	 * @see org.jfree.data.KeyedValues#getIndex(java.lang.Comparable)
	 */
	@SuppressWarnings("rawtypes")
	public int getIndex(Comparable key) {
		return keys.indexOf(key);
	}

	/**
	 * @see org.jfree.data.KeyedValues#getKeys()
	 */
	@SuppressWarnings("rawtypes")
	public List getKeys() {
		return Collections.unmodifiableList(keys);
	}

	/**
	 * @see org.jfree.data.KeyedValues#getValue(java.lang.Comparable)
	 */
	@SuppressWarnings("rawtypes")
	public Number getValue(Comparable key) {
		return data.get(key);
	}

	/**
	 * @see org.jfree.data.Values#getItemCount()
	 */
	public int getItemCount() {
		return keys.size();
	}

	/**
	 * @see org.jfree.data.Values#getValue(int)
	 */
	public Number getValue(int index) {
		String key = keys.get(index);
		return data.get(key);
	}

	/**
	 * @see org.jfree.data.general.Dataset#addChangeListener(org.jfree.data.general
	 *      .DatasetChangeListener)
	 */
	public void addChangeListener(DatasetChangeListener listener) {
		this.listener.add(listener);
	}

	/**
	 * @see org.jfree.data.general.Dataset#removeChangeListener(org.jfree.data.general
	 *      .DatasetChangeListener)
	 */
	public void removeChangeListener(DatasetChangeListener listener) {
		this.listener.remove(listener);
	}

	/**
	 * @see org.jfree.data.general.Dataset#getGroup()
	 */
	public DatasetGroup getGroup() {
		return group;
	}

	/**
	 * @see org.jfree.data.general.Dataset#setGroup(org.jfree.data.general.DatasetGroup
	 *      )
	 */
	public void setGroup(DatasetGroup group) {
		this.group = group;
	}

	public void add(Statistics st) {
		for (String key : st.keySet()) {
			if (!keys.contains(key)) {
				keys.add(key);
				colors.put(key, colorMapping.map("", key));
			}

			data.add(key, st.get(key));
		}

		this.fireDatasetChanged();
	}

	public void set(Statistics st) {
		keys.clear();
		data.clear();
		add(st);
	}

	public void fireDatasetChanged() {
		DatasetChangeEvent evt = new DatasetChangeEvent(this, this);
		for (DatasetChangeListener l : listener) {
			l.datasetChanged(evt);
		}
	}
}