package stream.dashboard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.Serializable;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.Data;
import stream.ProcessContext;
import stream.StatefulProcessor;
import stream.data.Statistics;
import stream.expressions.Condition;
import stream.util.KeyFilter;

public class Plot extends Widget implements StatefulProcessor {

	static Logger log = LoggerFactory.getLogger(Plot.class);

	/** The unique class ID */
	private static final long serialVersionUID = -5234179636369615078L;

	String timeKey = "@timestamp";
	String[] keys;
	String[] colors;

	Condition condition;
	PlotPanel plot;

	public Plot() {
		plot = new PlotPanel();
		plot.setKeys(this.keys);
	}

	public void init(ProcessContext ctx) throws Exception {
		super.init(ctx);
		this.plot.setKeys(keys);
		this.content.add(plot, BorderLayout.CENTER);

		if (colors != null && colors.length > 0) {
			Color[] cols = new Color[colors.length];
			for (int i = 0; i < colors.length; i++) {

				String color = colors[i];
				if (color == null)
					continue;

				int r = Integer.parseInt(color.substring(1, 3), 16);
				int g = Integer.parseInt(color.substring(3, 5), 16);
				int b = Integer.parseInt(color.substring(5), 16);

				Color c = new Color(r, g, b);
				cols[i] = c;
			}
			plot.setColors(cols);
		}
	}

	public Data process(Data input) {
		log.trace("Processing {}", input);

		Statistics st = new Statistics();

		Set<String> selected = KeyFilter.select(input, keys);
		// log.debug("Processing selected keys: {}", selected);
		for (String key : selected) {
			if (key.startsWith(Data.ANNOTATION_PREFIX))
				continue;
			Serializable val = input.get(key);
			if (val instanceof Number) {
				st.put(key, ((Number) val).doubleValue());
			}
		}
		plot.dataArrived(st);
		return input;
	}

	public String[] getKeys() {
		return keys;
	}

	public void setKeys(String[] keys) {
		this.keys = keys;
	}

	protected boolean containsKey(String key) {
		if (keys == null)
			return false;

		for (String k : keys) {
			if (k.equals(key))
				return true;
		}
		return false;
	}

	/**
	 * @return the condition
	 */
	public Condition getCondition() {
		return condition;
	}

	/**
	 * @param condition
	 *            the condition to set
	 */
	public void setCondition(Condition condition) {
		this.condition = condition;
	}

	/**
	 * @return the timeKey
	 */
	public String getTimeKey() {
		return timeKey;
	}

	/**
	 * @param timeKey
	 *            the timeKey to set
	 */
	public void setTimeKey(String timeKey) {
		this.timeKey = timeKey;
	}

	/**
	 * @return the colors
	 */
	public String[] getColors() {
		return colors;
	}

	/**
	 * @param colors
	 *            the colors to set
	 */
	public void setColors(String[] colors) {
		this.colors = colors;
	}

}