package stream.dashboard;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.geom.Arc2D;
import java.text.DecimalFormat;
import java.util.regex.Pattern;

import javax.swing.JFrame;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.SeriesRenderingOrder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.AbstractXYItemRenderer;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer2;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.xy.XYStepAreaRenderer;
import org.jfree.data.RangeType;
import org.jfree.ui.RectangleInsets;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.data.Statistics;
import stream.statistics.History;
import stream.statistics.StatisticsHistory;

public class TimelineChart extends Chart {
	static Logger log = LoggerFactory.getLogger(TimelineChart.class);
	public final static int AREA_CHART = 0;
	public final static int LINE_CHART = 1;

	public enum Type {
		LineChart, AreaChart, StepAreaChart, StackedAreaChart
	};

	Type linetype = Type.AreaChart;

	final StatisticsHistory statistics;
	final TimelineDatasetAdapter data;
	final XYPlot plot;
	AbstractXYItemRenderer renderer;

	public TimelineChart(long history, long step, Type lineType) {
		super("");
		statistics = new StatisticsHistory(step, history);
		data = new TimelineDatasetAdapter(statistics);

		linetype = lineType;
		this.plot = createPlot(data, statistics, ".*");
	}

	private XYPlot createPlot(final TimelineDatasetAdapter dataset,
			final StatisticsHistory stats, String filter) {

		DateAxis dateAxis = new DateAxis();
		dateAxis.setPositiveArrowVisible(true);
		dateAxis.setNegativeArrowVisible(true);
		dateAxis.setRightArrow(new Arc2D.Double(Arc2D.OPEN));
		dateAxis.setLowerMargin(0.0d);
		dateAxis.setUpperMargin(0.0d);
		dateAxis.setAutoRange(false);
		dateAxis.setMinorTickCount(stats.getSteps());
		// dateAxis.setMinimumDate(dataset.getStart());
		// dateAxis.setMaximumDate(dataset.getEnd());
		dateAxis.setLabelPaint(Color.white);
		//
		// if (stats.getStepSize() == StatisticsHistory.DAY) {
		// // log.info( "#1 => Setting time-format to HH:mm" );
		// dateAxis.setTickUnit(new DateTickUnit(DateTickUnit.DAY, 1));
		// dateAxis.setDateFormatOverride(new SimpleDateFormat("dd.MM."));
		// }
		//
		// if (stats.getStepSize() == StatisticsHistory.HOUR) {
		// // log.info( "#2 => Setting time-format to HH:mm" );
		// dateAxis.setTickUnit(new DateTickUnit(DateTickUnit.HOUR, 2));
		// dateAxis.setDateFormatOverride(new SimpleDateFormat("HH:mm"));
		// dateAxis.setAutoRange(true);
		// }
		//
		// if (stats.getStepSize() == StatisticsHistory.MINUTE) {
		// dateAxis.setTickUnit(new DateTickUnit(DateTickUnit.MINUTE, 10));
		// // log.info( "#3 => Setting time-format to HH:mm:ss " );
		// dateAxis.setDateFormatOverride(new SimpleDateFormat("HH:mm"));
		// }

		NumberAxis yAxis1 = new NumberAxis();

		yAxis1.setAutoRange(true);
		yAxis1.setRangeType(RangeType.POSITIVE);
		yAxis1.setAutoRangeIncludesZero(true);
		yAxis1.setNumberFormatOverride(new DecimalFormat("0"));

		renderer = new XYStepAreaRenderer();
		int i = 1;

		if (i == 0)
			renderer = new XYStepAreaRenderer();

		if (i == 1)
			renderer = new StackedXYAreaRenderer();

		if (i == 2)
			renderer = new StackedXYAreaRenderer2();

		renderer = new XYLineAndShapeRenderer();
		renderer = new XYAreaRenderer();

		if (linetype == Type.LineChart) {
			XYLineAndShapeRenderer lasr = new XYLineAndShapeRenderer();
			lasr.setBaseShapesVisible(false);
			renderer = lasr;
		}

		if (linetype == Type.AreaChart)
			renderer = new XYAreaRenderer();

		if (linetype == Type.StepAreaChart)
			renderer = new XYStepAreaRenderer();

		if (linetype == Type.StackedAreaChart)
			renderer = new StackedXYAreaRenderer();

		renderer.setAutoPopulateSeriesShape(false);
		renderer.setAutoPopulateSeriesOutlinePaint(false);
		renderer.setAutoPopulateSeriesFillPaint(false);
		renderer.setAutoPopulateSeriesPaint(false);
		renderer.setBaseOutlineStroke(new BasicStroke(1.0f));
		renderer.setBaseStroke(new BasicStroke(3.0f));

		renderer.setBasePaint(Color.LIGHT_GRAY);
		renderer.setBaseOutlinePaint(Color.DARK_GRAY);
		renderer.setBaseItemLabelsVisible(false);
		//
		for (int s = 0; s < 5; s++) {
			Color c = getColor(s);
			renderer.setSeriesFillPaint(s, c);
			renderer.setSeriesOutlinePaint(s, Color.BLUE);
			renderer.setSeriesPaint(s, c);
			renderer.setSeriesItemLabelsVisible(s, false);
			renderer.setSeriesShape(s, null);
		}

		try {
			if (filter != null) {
				Pattern.compile(filter);
				dataset.setSeriesFilter(filter);
			} else
				dataset.setSeriesFilter(".*");
		} catch (Exception e) {
			e.printStackTrace();
		}

		XYPlot plot = new XYPlot(dataset, new DateAxis(), yAxis1, renderer);

		plot.setSeriesRenderingOrder(SeriesRenderingOrder.FORWARD);
		plot.setBackgroundPaint(Color.WHITE);
		plot.setOutlineVisible(false);
		plot.setRenderer(renderer);
		plot.setAxisOffset(new RectangleInsets(4.0d, 1.0d, 0.0d, 0.0d));
		return plot;
	}

	public void add(Statistics st) {
		add(System.currentTimeMillis(), st);
	}

	public void add(Long ts, Statistics st) {

		for (String key : st.keySet()) {
			int idx = data.seriesKeys.indexOf(key);

			if (idx < 0) {
				idx = data.seriesKeys.size();
				Color c = getColor(key);
				if (c == null)
					c = this.getColor(idx);
				log.info("Color for new series '{}' will be: {}", key, c);
				renderer.setSeriesFillPaint(idx, c);
				renderer.setSeriesOutlinePaint(idx, c);
				plot.setRenderer(renderer);
			}
		}

		data.add(ts, st);
	}

	public XYPlot getPlot() {
		return plot;
	}

	public static void main(String args[]) throws Exception {

		TimelineChart chart = new TimelineChart(History.MINUTE, History.SECOND,
				Type.AreaChart);

		JFreeChart jfc = new JFreeChart(chart.plot);
		ChartPanel p = new ChartPanel(jfc);

		JFrame frame = new JFrame();
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(p, BorderLayout.CENTER);
		frame.setSize(640, 240);
		frame.setVisible(true);

		while (true) {

			Statistics st = new Statistics("test");
			st.put("ABC", Math.random() * 10);
			chart.add(st);
			log.info("Adding new data...{}", st);
			Thread.sleep(1000);
		}
	}
}